from stripe import api_requestor, util
from stripe.api_resources.abstract.searchable_api_resource import (
    SearchableAPIResource,
)


async def _search_patch(
    cls,
    search_url,
    api_key=None,
    stripe_version=None,
    stripe_account=None,
    **params
):
    requestor = api_requestor.APIRequestor(
        api_key,
        api_base=cls.api_base(),
        api_version=stripe_version,
        account=stripe_account,
    )
    response, api_key = await requestor.request("get", search_url, params)
    stripe_object = util.convert_to_stripe_object(
        response, api_key, stripe_version, stripe_account
    )
    stripe_object._retrieve_params = params
    return stripe_object


SearchableAPIResource._search = classmethod(_search_patch)



import stripe
from async_stripe.api_resources.abstract import patch_custom_methods
from stripe import util


async def retrieve_features_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/features"
    headers = util.populate_headers(idempotency_key)
    resp = await self.request("get", url, params, headers)
    stripe_object = util.convert_to_stripe_object(resp)
    return stripe_object

async def update_features_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/features"
    headers = util.populate_headers(idempotency_key)
    resp = await self.request("post", url, params, headers)
    stripe_object = util.convert_to_stripe_object(resp)
    return stripe_object


stripe.treasury.FinancialAccount.retrieve_features = retrieve_features_patch
stripe.treasury.FinancialAccount.update_features = update_features_patch


custom_resources = [
    {"name": "retrieve_features", "http_verb": "get", "http_path": "features"},
    {"name": "update_features", "http_verb": "post", "http_path": "features"},
]
patch_custom_methods(stripe.treasury.FinancialAccount, custom_resources)



import stripe
from stripe import util
from async_stripe.api_resources.abstract import patch_custom_methods


async def submit_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/submit"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self


stripe.issuing.Dispute.submit = submit_patch


custom_methods = [
    {"name": "submit", "http_verb": "post"}
]

patch_custom_methods(stripe.issuing.Dispute, custom_methods)


import stripe
from stripe import api_requestor, six, util
from stripe.stripe_object import StripeObject

from stripe.six.moves.urllib.parse import quote_plus


async def search_patch(
    self, api_key=None, stripe_version=None, stripe_account=None, **params
):
    stripe_object = await self._request(
        "get",
        self.get("url"),
        api_key=api_key,
        stripe_version=stripe_version,
        stripe_account=stripe_account,
        **params
    )
    stripe_object._retrieve_params = params
    return stripe_object


async def _request_patch(
    self,
    method_,
    url_,
    api_key=None,
    idempotency_key=None,
    stripe_version=None,
    stripe_account=None,
    **params
):
    api_key = api_key or self.api_key
    stripe_version = stripe_version or self.stripe_version
    stripe_account = stripe_account or self.stripe_account

    requestor = api_requestor.APIRequestor(
        api_key, api_version=stripe_version, account=stripe_account
    )
    headers = util.populate_headers(idempotency_key)
    response, api_key = await requestor.request(method_, url_, params, headers)
    stripe_object = util.convert_to_stripe_object(
        response, api_key, stripe_version, stripe_account
    )
    return stripe_object


async def auto_paging_iter_patch(self):
    page = self

    while True:
        for item in page:
            yield item
        page = await page.next_search_result_page()

        if page.is_empty:
            break


async def next_search_result_page_patch(
    self, api_key=None, stripe_version=None, stripe_account=None, **params
):
    if not self.has_more:
        return self.empty_search_result(
            api_key=api_key,
            stripe_version=stripe_version,
            stripe_account=stripe_account,
        )

    params_with_filters = self._retrieve_params.copy()
    params_with_filters.update({"page": self.next_page})
    params_with_filters.update(params)

    return await self.search(
        api_key=api_key,
        stripe_version=stripe_version,
        stripe_account=stripe_account,
        **params_with_filters
    )


stripe.SearchResultObject.search = search_patch
stripe.SearchResultObject._request = _request_patch
stripe.SearchResultObject.auto_paging_iter = auto_paging_iter_patch
stripe.SearchResultObject.next_search_result_page = next_search_result_page_patch



import stripe
from stripe import util
from async_stripe.api_resources.abstract import patch_custom_methods, patch_nested_resources


async def expire_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/expire"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self


stripe.checkout.Session.expire = expire_patch


custom_resources = [
    {"name": "expire", "http_verb": "post"},
]
patch_custom_methods(stripe.checkout.Session, custom_resources)


nested_resources = ["line_item"]
patch_nested_resources(stripe.checkout.Session, nested_resources)


import stripe
from stripe import util
from async_stripe.api_resources.abstract import patch_custom_methods


async def cancel_action_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/cancel_action"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self

async def process_payment_intent_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/process_payment_intent"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self

async def process_setup_intent_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/process_setup_intent"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self

async def set_reader_display_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/set_reader_display"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self


stripe.terminal.Reader.cancel_action = cancel_action_patch
stripe.terminal.Reader.process_payment_intent = process_setup_intent_patch
stripe.terminal.Reader.process_setup_intent = process_setup_intent_patch
stripe.terminal.Reader.set_reader_display = set_reader_display_patch


custom_resources = [
    {"name": "cancel_action", "http_verb": "post"},
    {"name": "process_payment_intent", "http_verb": "post"},
    {"name": "process_setup_intent", "http_verb": "post"},
    {"name": "set_reader_display", "http_verb": "post"},
]
patch_custom_methods(stripe.terminal.Reader, custom_resources)


# methods for TestHelpers class (nested inside Reader class)
async def TestHelpers_present_payment_method_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/present_payment_method"
    headers = util.populate_headers(idempotency_key)
    self.resource.refresh_from(
        await self.resource.request("post", url, params, headers)
    )
    return self.resource


stripe.terminal.Reader.TestHelpers.present_payment_method = TestHelpers_present_payment_method_patch


TestHelpers_custom_resources = [
    {"name": "present_payment_method", "http_verb": "post"}
]
patch_custom_methods(stripe.terminal.Reader.TestHelpers, TestHelpers_custom_resources)



from stripe import error, util, six
from stripe.six.moves.urllib.parse import quote_plus
from stripe.api_resources.abstract.test_helpers import APIResourceTestHelpers
from async_stripe.api_resources.abstract.api_resource import APIResource


# Some resource classes have a nested TestHelpers class
# but they still have some unpatched methods
for subclass in APIResourceTestHelpers.__subclasses__():
    subclass._static_request = subclass._resource_cls._static_request
    subclass._static_request_stream = subclass._resource_cls._static_request_stream



import stripe
from async_stripe.api_resources.abstract import patch_custom_methods
from stripe import util


async def disconnect_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/disconnect"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self

async def refresh_account_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/refresh"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self

async def list_owners_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/owners"
    headers = util.populate_headers(idempotency_key)
    resp = await self.request("get", url, params, headers)
    stripe_object = util.convert_to_stripe_object(resp)
    stripe_object._retrieve_params = params
    return stripe_object


stripe.financial_connections.Account.disconnect = disconnect_patch
stripe.financial_connections.Account.refresh_account= refresh_account_patch
stripe.financial_connections.Account.list_owners = list_owners_patch


custom_resources = [
    {"name": "disconnect", "http_verb": "post", "http_path": "disconnect"},
    {"name": "refresh_account", "http_verb": "post", "http_path": "refresh"},
    {"name": "list_owners", "http_verb": "get", "http_path": "owners"},
]
patch_custom_methods(stripe.financial_connections.Account, custom_resources)



from async_stripe.api_resources.treasury import financial_account
from async_stripe.api_resources.treasury import inbound_transfer
from async_stripe.api_resources.treasury import outbound_payment
from async_stripe.api_resources.treasury import outbound_transfer
from async_stripe.api_resources.treasury import received_credit
from async_stripe.api_resources.treasury import received_debit



import stripe
from stripe import util
from async_stripe.api_resources.abstract import patch_custom_methods


async def pay_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/pay"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self


async def return_order_patch(self, idempotency_key=None, **params):
    headers = util.populate_headers(idempotency_key)
    return await self.request(
        "post", self.instance_url() + "/returns", params, headers
    )


async def cancel_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/cancel"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self

async def list_line_items_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/line_items"
    headers = util.populate_headers(idempotency_key)
    resp = await self.request("get", url, params, headers)
    stripe_object = util.convert_to_stripe_object(resp)
    stripe_object._retrieve_params = params
    return stripe_object

async def reopen_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/reopen"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self

async def submit_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/submit"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self


stripe.Order.cancel = cancel_patch
stripe.Order.list_line_items = list_line_items_patch
stripe.Order.reopen = reopen_patch
stripe.Order.submit = submit_patch


custom_methods = [
    {"name": "cancel", "http_verb": "post"},
    {"name": "list_line_items", "http_verb": "get", "http_path": "line_items"},
    {"name": "reopen", "http_verb": "post"},
    {"name": "submit", "http_verb": "post"},
]

patch_custom_methods(stripe.Order, custom_methods)



import stripe
from async_stripe.api_resources.abstract import (
    patch_nested_resources, patch_custom_methods
)
from stripe import api_requestor
from stripe import util


async def delete_discount_patch(self, **params):
    requestor = api_requestor.APIRequestor(
        self.api_key,
        api_version=self.stripe_version,
        account=self.stripe_account,
    )
    url = self.instance_url() + "/discount"
    _, api_key = await requestor.request("delete", url, params)
    self.refresh_from({"discount": None}, api_key, True)

async def list_payment_methods_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/payment_methods"
    headers = util.populate_headers(idempotency_key)
    resp = await self.request("get", url, params, headers)
    stripe_object = util.convert_to_stripe_object(resp)
    stripe_object._retrieve_params = params
    return stripe_object

async def create_funding_instructions_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/funding_instructions"
    headers = util.populate_headers(idempotency_key)
    resp = await self.request("post", url, params, headers)
    stripe_object = util.convert_to_stripe_object(resp)
    return stripe_object

async def _cls_retrieve_payment_method_patch(
    cls,
    customer,
    payment_method,
    api_key=None,
    stripe_version=None,
    stripe_account=None,
    **params
):
    requestor = api_requestor.APIRequestor(
        api_key, api_version=stripe_version, account=stripe_account
    )
    url = (
        "/v1/customers/{customer}/payment_methods/{payment_method}".format(
            customer=util.sanitize_id(customer),
            payment_method=util.sanitize_id(payment_method),
        )
    )
    response, api_key = await requestor.request("get", url, params)
    return util.convert_to_stripe_object(
        response, api_key, stripe_version, stripe_account
    )

@util.class_method_variant("_cls_retrieve_payment_method")
async def retrieve_payment_method_patch(
    self, payment_method, idempotency_key=None, **params
):
    url = (
        "/v1/customers/{customer}/payment_methods/{payment_method}".format(
            customer=util.sanitize_id(self.get("id")),
            payment_method=util.sanitize_id(payment_method),
        )
    )
    headers = util.populate_headers(idempotency_key)
    resp = await self.request("get", url, params, headers)
    stripe_object = util.convert_to_stripe_object(resp)
    return stripe_object

async def retrieve_cash_balance_patch(
    cls,
    customer,
    nested_id=None,
    api_key=None,
    stripe_version=None,
    stripe_account=None,
    **params
):
    # The nested_id parameter is required for backwards compatibility purposes and is ignored.
    requestor = api_requestor.APIRequestor(
        api_key, api_version=stripe_version, account=stripe_account
    )
    url = "/v1/customers/{customer}/cash_balance".format(
        customer=util.sanitize_id(customer)
    )
    response, api_key = await requestor.request("get", url, params)
    return util.convert_to_stripe_object(
        response, api_key, stripe_version, stripe_account
    )

async def modify_cash_balance_patch(
    cls,
    customer,
    nested_id=None,
    api_key=None,
    stripe_version=None,
    stripe_account=None,
    **params
):
    # The nested_id parameter is required for backwards compatibility purposes and is ignored.
    requestor = api_requestor.APIRequestor(
        api_key, api_version=stripe_version, account=stripe_account
    )
    url = "/v1/customers/{customer}/cash_balance".format(
        customer=util.sanitize_id(customer)
    )
    response, api_key = await requestor.request("post", url, params)
    return util.convert_to_stripe_object(
        response, api_key, stripe_version, stripe_account
    )


stripe.Customer.delete_discount = delete_discount_patch
stripe.Customer.list_payment_methods = list_payment_methods_patch
stripe.Customer._cls_retrieve_payment_method = classmethod(_cls_retrieve_payment_method_patch)
stripe.Customer.retrieve_payment_method = retrieve_payment_method_patch
stripe.Customer.retrieve_cash_balance = classmethod(retrieve_cash_balance_patch)
stripe.Customer.modify_cash_balance = classmethod(modify_cash_balance_patch)


netsted_resources = ['balance_transaction', 'source', 'tax_id']
patch_nested_resources(stripe.Customer, netsted_resources)


# always patch custom methods after patching other methods
custom_resources = [
    {"name": "delete_discount", "http_verb": "delete", "http_path": "discount"},
    {"name": "list_payment_methods", "http_verb": "get", "http_path": "payment_methods"},
    {"name": "create_funding_instructions", "http_verb": "post", "http_path": "funding_instructions"},
]
patch_custom_methods(stripe.Customer, custom_resources)



import stripe
from stripe import api_requestor
from stripe import util


# methods for TestHelpers nested class

async def create_patch(
    cls,
    api_key=None,
    stripe_version=None,
    stripe_account=None,
    **params
):
    requestor = api_requestor.APIRequestor(
        api_key, api_version=stripe_version, account=stripe_account
    )
    url = "/v1/test_helpers/treasury/received_debits"
    response, api_key = await requestor.request("post", url, params)
    return util.convert_to_stripe_object(
        response, api_key, stripe_version, stripe_account
    )



stripe.treasury.ReceivedDebit.TestHelpers.create = classmethod(create_patch)



import stripe
from stripe import util
from async_stripe.api_resources.abstract import patch_custom_methods


async def cancel_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/cancel"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self


stripe.Refund.cancel = cancel_patch


custom_resources = [
    {"name": "cancel", "http_verb": "post"}
]
patch_custom_methods(stripe.Refund, custom_resources)


# methods for TestHelpers nested class
async def TestHelpers_expire_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/expire"
    headers = util.populate_headers(idempotency_key)
    self.resource.refresh_from(
        self.resource.request("post", url, params, headers)
    )
    return self.resource


stripe.Refund.TestHelpers.expire = TestHelpers_expire_patch


TestHelpers_custom_resources = [
    {"name": "expire", "http_verb": "post"}
]
patch_custom_methods(stripe.Refund.TestHelpers, TestHelpers_custom_resources)



import stripe
from stripe import util
from async_stripe.api_resources.abstract import patch_custom_methods


async def list_line_items_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/line_items"
    headers = util.populate_headers(idempotency_key)
    resp = await self.request("get", url, params, headers)
    stripe_object = util.convert_to_stripe_object(resp)
    stripe_object._retrieve_params = params
    return stripe_object


stripe.PaymentLink.list_line_items = list_line_items_patch


custom_resources = [
    {"name": "list_line_items", "http_verb": "get", "http_path": "line_items"},
]
patch_custom_methods(stripe.PaymentLink, custom_resources)



import stripe
from async_stripe.api_resources.abstract import patch_custom_methods
from stripe import util


async def cancel_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/cancel"
    headers = util.populate_headers(idempotency_key)
    self.refresh_from(await self.request("post", url, params, headers))
    return self


stripe.treasury.OutboundTransfer.cancel = cancel_patch

custom_resources = [
    {"name": "cancel", "http_verb": "post"},
]
patch_custom_methods(stripe.treasury.OutboundTransfer, custom_resources)


# methods for TestHelpers nested class

async def fail_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/fail"
    headers = util.populate_headers(idempotency_key)
    self.resource.refresh_from(
        await self.resource.request("post", url, params, headers)
    )
    return self.resource

async def post_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/post"
    headers = util.populate_headers(idempotency_key)
    self.resource.refresh_from(
        await self.resource.request("post", url, params, headers)
    )
    return self.resource

async def return_outbound_transfer_patch(self, idempotency_key=None, **params):
    url = self.instance_url() + "/return"
    headers = util.populate_headers(idempotency_key)
    self.resource.refresh_from(
        await self.resource.request("post", url, params, headers)
    )
    return self.resource


stripe.treasury.OutboundTransfer.TestHelpers.fail = fail_patch
stripe.treasury.OutboundTransfer.TestHelpers.post = post_patch
stripe.treasury.OutboundTransfer.TestHelpers.return_outbound_transfer = return_outbound_transfer_patch


TestHelpers_custom_resources = [
    {"name": "fail", "http_verb": "post"},
    {"name": "post", "http_verb": "post"},
    {"name": "return_outbound_transfer", "http_verb": "post", "http_path": "return"},
]
patch_custom_methods(stripe.treasury.OutboundTransfer.TestHelpers, TestHelpers_custom_resources)
